@@ -1,25 +1,27 @@
 /*
-  Copyright (c) 2013-2017 Montel Laurent <montel@kde.org>
+  Copyright (c) 2013-2018 Montel Laurent <montel@kde.org>
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License, version 2, as
-  published by the Free Software Foundation.
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Lesser General Public License for more details.
 
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "grantleetheme.h"
 #include "grantleetheme_p.h"
 #include "grantleetheme_debug.h"
 #include "grantleethemeengine.h"
-#include "config-grantleetheme.h"
+#include <config-grantleetheme.h>
+#include "qtresourcetemplateloader.h"
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -67,7 +69,7 @@ void ThemePrivate::setupLoader()
     QDir dir(absolutePath);
     dir.cdUp();
 
-    loader = QSharedPointer<Grantlee::FileSystemTemplateLoader>::create();
+    loader = QSharedPointer<GrantleeTheme::QtResourceTemplateLoader>::create();
     loader->setTemplateDirs({ dir.absolutePath() });
     loader->setTheme(dirName);
 
@@ -88,21 +90,22 @@ Grantlee::Context ThemePrivate::createContext(const QVariantHash &data, const QB
     return ctx;
 }
 
-QString ThemePrivate::errorTemplate(const QString &reason,
-                                    const QString &origTemplateName,
-                                    const Grantlee::Template &failedTemplate)
+QString ThemePrivate::errorTemplate(const QString &reason, const QString &origTemplateName, const Grantlee::Template &failedTemplate)
 {
     Grantlee::Template tpl = sEngine->newTemplate(
-                                 QStringLiteral("<h1>{{ error }}</h1>\n"
-                                         "<b>%1:</b> {{ templateName }}<br>\n"
-                                         "<b>%2:</b> {{ errorMessage }}")
-                                 .arg(i18n("Template"), i18n("Error message")),
-                                 QStringLiteral("TemplateError"));
+        QStringLiteral("<h1>{{ error }}</h1>\n"
+                       "<b>%1:</b> {{ templateName }}<br>\n"
+                       "<b>%2:</b> {{ errorMessage }}")
+        .arg(i18n("Template"), i18n("Error message")),
+        QStringLiteral("TemplateError"));
 
     Grantlee::Context ctx = createContext();
     ctx.insert(QStringLiteral("error"), reason);
     ctx.insert(QStringLiteral("templateName"), origTemplateName);
-    ctx.insert(QStringLiteral("errorMessage"), failedTemplate->errorString());
+    const QString errorString = failedTemplate
+                                ? failedTemplate->errorString()
+                                : i18n("(null template)");
+    ctx.insert(QStringLiteral("errorMessage"), errorString);
     return tpl->render(&ctx);
 }
 
@@ -208,7 +211,7 @@ QString Theme::render(const QString &templateName, const QVariantHash &data, con
     }
 
     Grantlee::Template tpl = d->loader->loadByName(templateName, ThemePrivate::sEngine);
-    if (tpl->error()) {
+    if (!tpl || tpl->error()) {
         return d->errorTemplate(i18n("Template parsing error"), templateName, tpl);
     }
 
